//
//  WorkoutManager.swift
//  CapstoneProject WatchKit Extension
//
//  Created by Easten Wember on 2/25/22.
//
//manage the selected workout (which is an option HKWorkoutActivityType)

import Foundation
import HealthKit //want healthKit APIs!!
import OSLog
import UserNotifications

class WorkoutManager: NSObject, ObservableObject {
    
    var selectedWorkout: HKWorkoutActivityType? { //track selected workout
        //whenever selectedWorkoutChanges, want to call startWorkout
        didSet { //selectedWorkout can be null, so use guard to change selectedWorkout only when valid
            guard let selectedWorkout = selectedWorkout else { return }
            startWorkout(workoutType: selectedWorkout)
        }
    }

    @Published var showingSummaryView: Bool = false { //provide binding to a sheet's selection on apps NavigationView
        didSet {
            //sheet dismissed
            if showingSummaryView == false {
                resetWorkout() //reset the workout
            }
        }
    }

    let healthStore = HKHealthStore()
    var session: HKWorkoutSession?
    var builder: HKLiveWorkoutBuilder?

    // Start the workout!
    func startWorkout(workoutType: HKWorkoutActivityType) {
        let configuration = HKWorkoutConfiguration()
        configuration.activityType = workoutType
        configuration.locationType = .unknown //could be indoor/outdoor...impacts accurarcy of some data collection

        //create session and obtain the workout builder
        do { //create HKWorkoutSession
            session = try HKWorkoutSession(healthStore: healthStore, configuration: configuration)
            builder = session?.associatedWorkoutBuilder()
        } catch {
            print("here exception thrown?")
            return //handle excetptions here
        }

        //setup session and builder
        session?.delegate = self //assign workoutManager as the HKworkoutSession delegate
        builder?.delegate = self

        //**!!HKLiveWorkoutDataSource provides live data from an active workout session
        builder?.dataSource = HKLiveWorkoutDataSource(healthStore: healthStore, //assign builder's datasource to an HKLiveWorkoutDataSurce
                                                     workoutConfiguration: configuration)

        //start session and begin data collection
        let startDate = Date()
        session?.startActivity(with: startDate)
        builder?.beginCollection(withStart: startDate) { (success, error) in
            //workout has started
        }
    }

    // request authorization to access HealthKit
    func requestAuthorization() {
        //quantity type to write to the health store
        let typesToShare: Set = [
            HKQuantityType.workoutType()
        ]

        // The quantity types to read from the health store.
        let typesToRead: Set = [
            HKQuantityType.quantityType(forIdentifier: .heartRate)!,
            HKQuantityType.quantityType(forIdentifier: .restingHeartRate)!,
            HKQuantityType.quantityType(forIdentifier: .activeEnergyBurned)!,
            HKQuantityType.quantityType(forIdentifier: .distanceWalkingRunning)!,
            //HKQuantityType.quantityType(forIdentifier: .walkingStepLength)!,
            HKQuantityType.quantityType(forIdentifier: .stepCount)!,
            HKQuantityType.quantityType(forIdentifier: .respiratoryRate)!, //how many breaths per min
            HKQuantityType.quantityType(forIdentifier: .appleStandTime)!,
            HKQuantityType.quantityType(forIdentifier: .appleMoveTime)!,
            //HKObserverQuery.EventType
            
            HKObjectType.activitySummaryType() //want permission to read the activity rings summary
        ]

        //request authorization for those quantity types
        healthStore.requestAuthorization(toShare: typesToShare, read: typesToRead) { (success, error) in
            //handle error
        }
    }

    //State Control

    //the workout session state
    @Published var running = false //track if session is running

    func togglePause() { //pause or resume session based on current state
        if running == true {
            self.pause()
        } else {
            resume()
        }
    }

    func pause() {
        session?.pause()
    }

    func resume() {
        session?.resume()
    }

    func endWorkout() {
        session?.end()
        showingSummaryView = true
    }

    //workout metrics
    @Published var averageHeartRate: Double = 0
    @Published var heartRate: Double = 0
    @Published var activeEnergy: Double = 0
    @Published var distance: Double = 0
    @Published var workout: HKWorkout?
    
    //*may need to add here ? to calculate some ..use to display in metricsView OR to push notifications
    //HKQauntityTypeIdentifiers...not sure what their types are??
    @Published var stepCount: Double = 0 //added  ? bc i think stepcount is int? but need to use DoubleValue to grab HKvalue stepCount
    @Published var stepLength: Double? = 0
    @Published var respRate : Double = 0
    @Published var restHR : Double = 0 //comp to current HR/AVG
    @Published var standTime : Int = 0
    
    //var to control rate of notification...for now
    @Published var countHRNotifs : Int = 20
    
    func getStepCount(_ statistics: HKStatistics?){ //doesnt work...just returns same thing as distance??
        print("here in getStepCount")
        guard let statistics = statistics else { return } //unwraps statisitcs
        
        let stepCount = HKUnit.meter()
        self.stepCount = statistics.sumQuantity()?.doubleValue(for: stepCount) ?? 0
        print("step count: \(self.stepCount)")
        
        //get stepLength...not sure what the units are on this bc big...not useful for now
//        let stepLength = HKUnit.foot()
//        self.stepLength = (statistics.sumQuantity()?.doubleValue(for: stepLength))
//        //self.stepLength = (statistics.sumQuantity()?.doubleValue(for: stepLength)) ?? 0 //attempt to update steplength
//        print("step length: \(self.stepLength ?? 0)") //default value 0 if step length is nothing

    }
    
    
    func getRespRate(_ statistics: HKStatistics?){ //work on this...see if can grab a diff way?? HKUnit is wrong!! keeps causing error
        guard let statistics = statistics else { return } //unwraps statisitcs
        
        let respRate = HKUnit.count() //count be .count? thouight like Beat per minute
        self.respRate = statistics.averageQuantity()?.doubleValue(for: respRate) ?? 0
        print("in func resp rate: \(self.respRate)")
    }
    
    
    
    

    func updateForStatistics(_ statistics: HKStatistics?) {
        
        guard let statistics = statistics else { return }

        DispatchQueue.main.async {
            //print("hello")
            switch statistics.quantityType {
                //get HR#imageLiteral(resourceName: "04122001.heic")
            case HKQuantityType.quantityType(forIdentifier: .heartRate):
                let heartRateUnit = HKUnit.count().unitDivided(by: HKUnit.minute())
                self.heartRate = statistics.mostRecentQuantity()?.doubleValue(for: heartRateUnit) ?? 0
                self.averageHeartRate = statistics.averageQuantity()?.doubleValue(for: heartRateUnit) ?? 0
                print("curr HR: \(self.heartRate)")
                self.heartRateNotification()
                //self.getStepCount(statistics) //attempt to grab stepLength here
                //self.getRespRate(statistics) //call respRate function
                

                //get cals burned
            case HKQuantityType.quantityType(forIdentifier: .activeEnergyBurned):
                let energyUnit = HKUnit.kilocalorie()
                self.activeEnergy = statistics.sumQuantity()?.doubleValue(for: energyUnit) ?? 0
                print("cal burned: \(self.activeEnergy)")

                
                //get distance travelled
            case HKQuantityType.quantityType(forIdentifier: .distanceWalkingRunning): //, HKQuantityType.quantityType(forIdentifier: .distanceCycling)
                let meterUnit = HKUnit.meter()
                self.distance = statistics.sumQuantity()?.doubleValue(for: meterUnit) ?? 0
                print("distance travelled: \(self.distance)")
                self.getStepCount(statistics) //attempt to grab stepLength here
                
            
                //attempt to get stepLength
            //case HKQuantityType.quantityType(forIdentifier: .walkingStepLength):
                    //self.getStepLength(statistics)
                
            //attempt to get respiaroy rate
//            case HKQuantityType.quantityType(forIdentifier: .respiratoryRate):
//                let respRate = HKUnit.minute() //count be .count? thouight like Beat per minute
//                self.respRate = statistics.averageQuantity()?.doubleValue(for: respRate) ?? 0
//                print("in case resp rate: \(self.respRate)")
                
                
                //get step count...couldnt get to wokr
//            case HKQuantityType.quantityType(forIdentifier: .stepCount): //attempt to update stepCount
//                print("here in step count")
//                let stepCount = HKUnit.meter()
//                self.stepCount = statistics.sumQuantity()?.doubleValue(for: stepCount) ?? 0
//                print("step count: \(self.stepCount ?? 0)")

            default:
                return
            }
        } //end switch
    }

    func resetWorkout() {
        selectedWorkout = nil
        builder = nil
        workout = nil
        session = nil
        activeEnergy = 0
        averageHeartRate = 0
        heartRate = 0
        distance = 0
    }
    
    
    func heartRateNotification() { //left off here..enters but no notification show on watch.....doesnt work
        print("in heartRateNotification..count: \(countHRNotifs)")

        if countHRNotifs == 20{
            print("should send notification to user...")
            let center = UNUserNotificationCenter.current()

            let content = UNMutableNotificationContent()
            content.title = "Stress Update"
            content.subtitle = "Avg HR is: \(self.averageHeartRate)"
            
            if heartRate > averageHeartRate{
                content.body = "current HR \(self.heartRate) above avg!"
            }
            else{
                content.body = "current HR \(self.heartRate) below avg!"
            }
            
        
            content.categoryIdentifier = "alarm"
            content.userInfo = ["customData": "fizzbuzz"]
            content.sound = UNNotificationSound.default

            //deliver notification after specified time interval...60 seconds * 1 for one minute..
            //notification only goes once ...repeats:false
            let trigger = UNTimeIntervalNotificationTrigger(timeInterval: (1*60), repeats: false)

            let request = UNNotificationRequest(identifier: UUID().uuidString, content: content, trigger: trigger)
            center.add(request)
            
            //**EVENTUALLY SOLVE THIS WITH TIME INTERVAL/REPEATS on timeintervalnotification trigger
            countHRNotifs = 0 //reset count to space out notifications user gets
        }
        else{
            countHRNotifs += 1 //work way back up to 20
        }
        
    }//end heartRateNotification
    
    
}//end workout manager

//extend HKWorkoutSessionDelegate...listen to changes in session state
extension WorkoutManager: HKWorkoutSessionDelegate {
    func workoutSession(_ workoutSession: HKWorkoutSession, didChangeTo toState: HKWorkoutSessionState, //called whenever session state changes
                        from fromState: HKWorkoutSessionState, date: Date) {
        DispatchQueue.main.async {
            self.running = toState == .running
        }

        // Wait for the session to transition states before ending the builder.
        if toState == .ended {
            builder?.endCollection(withEnd: date) { (success, error) in
                self.builder?.finishWorkout { (workout, error) in
                    DispatchQueue.main.async { //running variable updates based on if toState is running + is dispatched to main queue for UI updates
                        self.workout = workout
                    }
                }
            }
        }
    }

    func workoutSession(_ workoutSession: HKWorkoutSession, didFailWithError error: Error) {

    }
}

//HKLiveWorkoutBuilderDelegate
extension WorkoutManager: HKLiveWorkoutBuilderDelegate {
    func workoutBuilderDidCollectEvent(_ workoutBuilder: HKLiveWorkoutBuilder) {

    }

    func workoutBuilder(_ workoutBuilder: HKLiveWorkoutBuilder, didCollectDataOf collectedTypes: Set<HKSampleType>) {
        for type in collectedTypes {
            guard let quantityType = type as? HKQuantityType else {
                return // Nothing to do.
            }

            let statistics = workoutBuilder.statistics(for: quantityType)

            // Update the published values.
            updateForStatistics(statistics)
        }
    }
}

